@@ -21,7 +21,8 @@ from fog.utils import OnlineMean
 def best_matching(
     truth: Iterable[Iterable[Hashable]],
     predicted: Iterable[Iterable[Hashable]],
-    allow_additional_items: bool = False
+    allow_additional_items: bool = False,
+    macro: bool = True
 ) -> Tuple[float, float, float]:
     """
     Efficient implementation of the "best matching F1" evaluation metric for
@@ -34,6 +35,8 @@ def best_matching(
             that don't exist in truth clusters to be found in predicted ones. Those
             additional items will then be ignored when computing the metrics instead
             of raising an error when found. Defaults to False.
+        macro (bool, optional): Whether to compute the macro or the micro average
+            of the evaluation metric. Defaults to True.
 
     Returns:
         tuple of floats: precision, recall and f1 score.
@@ -44,7 +47,16 @@ def best_matching(
     index = {}
     truth_cluster_sizes = {}
 
+    # NOTE: I use a counter because the function accepts arbitrary (potentially lazy)
+    # iterables, not just sized ones.
+    c = 0
+
     for i, cluster in enumerate(truth):
+        c += 1
+
+        if not cluster:
+            raise TypeError('truth contains an empty cluster')
+
         for item in cluster:
             if item in index:
                 raise TypeError('truth clusters are fuzzy (i.e. one item can be found in multiple clusters)')
@@ -53,12 +65,25 @@ def best_matching(
 
         truth_cluster_sizes[i] = len(cluster)
 
+    if c == 0:
+        raise TypeError('truth is empty')
+
     # Aggregating scores
     P = OnlineMean()
     R = OnlineMean()
     F = OnlineMean()
 
+    micro_true_positives = 0
+    micro_false_positives = 0
+    micro_false_negatives = 0
+
+    c = 0
+
     for cluster in predicted:
+        c += 1
+
+        if not cluster:
+            raise TypeError('predicted contains an empty cluster')
 
         # Finding best matching cluster from truth
         candidates = Counter()
@@ -86,16 +111,35 @@ def best_matching(
         false_positives = cluster_size - true_positives
         false_negatives = matching_cluster_size - true_positives
 
-        precision = true_positives / (true_positives + false_positives)
-        recall = true_positives / (true_positives + false_negatives)
-        f1 = 2 * precision * recall / (precision + recall)
+        if macro:
+            precision = true_positives / (true_positives + false_positives)
+            recall = true_positives / (true_positives + false_negatives)
+            f1 = 2 * precision * recall / (precision + recall)
 
-        P.add(precision)
-        R.add(recall)
-        F.add(f1)
+            P.add(precision)
+            R.add(recall)
+            F.add(f1)
+
+        else:
+            micro_true_positives += true_positives
+            micro_false_positives += false_positives
+            micro_false_negatives += false_negatives
+
+    if c == 0:
+        raise TypeError('predicted is empty')
+
+    if macro:
+        return (
+            float(P),
+            float(R),
+            float(F)
+        )
+
+    micro_precision = micro_true_positives / (micro_true_positives + micro_false_positives)
+    micro_recall = micro_true_positives / (micro_true_positives + micro_false_negatives)
 
     return (
-        float(P),
-        float(R),
-        float(F)
+        micro_precision,
+        micro_recall,
+        2 * micro_precision * micro_recall / (micro_precision + micro_recall)
     )
