@@ -35,7 +35,7 @@ IRISH = ['O', 'o']
 VOWELS_PATTERN = 'aáàâäąåoôóøeéèëêęiíïîıuúùûüyÿæœAÁÀÂÄĄÅOÓÔØEÉÈËÊĘIİÍÏÎYŸUÚÙÛÜÆŒ'
 CONSONANTS_RE = re.compile(r'^[^%s]$' % VOWELS_PATTERN)
 EMAIL_LOOKAHEAD_RE = re.compile(r'^[A-Za-z0-9!#$%&*+\-/=?^_`{|}~]{1,64}@')
-SMILEY_RE = re.compile(r'^(?:[\-]+>|<[\-]+|[<>]?[:;=8][\-o\*\']?[\)\]\(\[dDpP/\:\}\{@\|\\]|[\)\]\(\[dDpP/\:\}\{@\|\\][\-o\*\']?[:;=8]|[<:]3)')
+SMILEY_RE = re.compile(r'^(?:[\-]+>|<[\-]+|[<>]?[:;=8][\-o\*\']?[\)\]\(\[dDpP/\:\}\{@\|\\]|[\)\]\(\[dDpP/\:\}\{@\|\\][\-o\*\']?[:;=8]|[<:]3|\^\^)')
 EMOJI_RE = get_emoji_regexp()
 POINT_SPLITTER_RE = re.compile(r'(\.)')
 LENGTHENING_RE = re.compile(r'(.)\1{2,}')
@@ -172,16 +172,66 @@ def validate_token_types(types):
 
 def split_hashtag(string):
     offset = 1
-
     it = enumerate(string)
-    next(it)
+    state = 'upper-start' if not string[1].isdigit() else 'number'
 
+    next(it)  # Hastag sigil '#'
+    next(it)  # First char
+
+    # NOTE: I hate state machines (I don't use a regex here to keep the benefit
+    # of python's string methods wrt diacritics)
     for i, c in it:
-        if c.isupper() and offset < i:
-            yield string[offset:i]
-            offset = i
+        if state == 'lower':
+            if c.isupper():
+                yield ('word', string[offset:i])
+                state = 'upper-start'
+                offset = i
 
-    yield string[offset:]
+            elif c.isdigit():
+                yield ('word', string[offset:i])
+                state = 'number'
+                offset = i
+
+        elif state == 'upper-start':
+            if c.islower():
+                state = 'lower'
+                continue
+
+            if c.isupper():
+                state = 'upper-next'
+                continue
+
+            if c.isdigit():
+                state = 'number'
+                yield ('word', string[offset:i])
+                offset = i
+
+        elif state == 'upper-next':
+            if c.islower():
+                state = 'lower'
+                yield ('word', string[offset:i - 1])
+                offset = i - 1
+
+            elif c.isdigit():
+                state = 'number'
+                yield ('word', string[offset:i])
+                offset = i
+
+        elif state == 'number':
+            if not c.isdigit():
+                if c.isupper():
+                    state = 'upper-start'
+                    yield ('number', string[offset:i])
+                    offset = i
+
+                else:
+                    state = 'lower'
+                    yield ('number', string[offset:i])
+                    offset = i
+
+        prev = c
+
+    yield ('number' if state == 'number' else 'word', string[offset:])
 
 
 class WordTokenizer(object):
@@ -411,7 +461,7 @@ class WordTokenizer(object):
                         if n == ',':
                             break
 
-                    elif n in IDENTIFIER_PARTS:
+                    elif n in IDENTIFIER_PARTS or n == '·':
                         if j + 1 >= l:
                             break
 
@@ -521,8 +571,7 @@ class WordTokenizer(object):
                         yield (token_type, before)
 
                 elif token_type == 'hashtag' and self.split_hashtags:
-                    for sub_token in split_hashtag(before):
-                        yield ('word', sub_token)
+                    yield from split_hashtag(before)
 
                 else:
 
